/*
 * Copyright Openmind http://www.openmindonline.it
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package it.openutils.spring.rmibernate.client;

import it.openutils.spring.rmibernate.shared.managers.HibernateLazyService;

import org.aopalliance.intercept.MethodInvocation;
import org.springframework.remoting.rmi.RmiProxyFactoryBean;


/**
 * ProxyFactoryBean for automatic injection of remote lazy loading proxies
 * @author mmolaschi
 * @version $Id: HibernateRmiProxyFactoryBean.java 716 2008-03-03 14:35:57Z fcarone $
 */
public class HibernateRmiProxyFactoryBean extends RmiProxyFactoryBean
{

    /**
     * Store the hibernate lazy service to be used by remote lazy loading
     */
    public static ThreadLocal<HibernateLazyService> hibernateLazyServiceTL = new ThreadLocal<HibernateLazyService>();

    private static final long serialVersionUID = 6545259150454309616L;

    private HibernateLazyService hibernateLazyService;

    /**
     * {@inheritDoc}
     */
    @Override
    public Object invoke(MethodInvocation invocation) throws Throwable
    {
        // store the lazy service for this thread
        if (hibernateLazyService == null)
        {
            // if hibernateLazyService is null this means that this service IS hibernateLazyService
            hibernateLazyServiceTL.set((HibernateLazyService) this.getObject());
        }
        else
        {
            hibernateLazyServiceTL.set(hibernateLazyService);
        }

        // proceed with rmi call
        return super.invoke(invocation);
    }

    /**
     * Sets the hibernateLazyService.
     * @param hibernateLazyService the hibernateLazyService to set
     */
    public void setHibernateLazyService(HibernateLazyService hibernateLazyService)
    {
        this.hibernateLazyService = hibernateLazyService;
    }

}
