/*
 * Copyright Openmind http://www.openmindonline.it
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package it.openutils.spring.rmibernate.client.aspects;

import it.openutils.spring.rmibernate.client.pagination.Paginator;
import it.openutils.spring.rmibernate.shared.managers.HibernateLazyService;

import java.io.Serializable;

import net.sf.cglib.proxy.LazyLoader;


/**
 * Aspect that does remote lazy loading on cglib proxy
 * @author mmolaschi
 * @version $Id: HibernateLazyLoaderAspect.java 716 2008-03-03 14:35:57Z fcarone $
 */
public class HibernateLazyLoaderAspect implements LazyLoader, Serializable
{

    /**
     * UID
     */
    private static final long serialVersionUID = -365731708075101363L;

    private static ThreadLocal<Paginator> paginator = new ThreadLocal<Paginator>();

    private String className;

    private String fieldName;

    private Serializable id;

    private Object localReference;

    private HibernateLazyService hibernateLazyService;

    /**
     * Constructor
     * @param className parent class name
     * @param fieldName field to be lazy loaded
     * @param id id of current entity
     */
    public HibernateLazyLoaderAspect(String className, String fieldName, Serializable id, HibernateLazyService hibernateLazyService)
    {
        this.className = className;
        this.fieldName = fieldName;
        this.id = id;
        this.hibernateLazyService = hibernateLazyService;
    }

    /**
     * Set pagination for next calls in current thread
     * @param from starting row number
     * @param size max number of rows
     */
    public static void setPagination(int from, int size)
    {
        paginator.set(new Paginator(from, size));
    }

    /**
     * Disable pagination for next calls in current thread
     */
    public static void resetPagination()
    {
        paginator.set(null);
    }

    /**
     * {@inheritDoc}
     */
    public Object loadObject() throws Exception
    {
        // @todo handle localreference with paginator
        // Load from remote
        if (paginator.get() != null)
        {
            return hibernateLazyService.invoke(className, fieldName, id, paginator.get().getFrom(), paginator.get().getSize());
        }
        if (localReference == null)
        {
            localReference = hibernateLazyService.invoke(className, fieldName, id);
        }
        return localReference;
    }

}
