/*
 * Copyright Openmind http://www.openmindonline.it
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package it.openutils.spring.rmibernate.server.aspects;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.hibernate.SessionFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Main server interceptor on rmi calls
 * @author mmolaschi
 * @version $Id: HibernateLazyRmiInterceptor.java 716 2008-03-03 14:35:57Z fcarone $
 */
public class HibernateLazyRmiInterceptor implements MethodInterceptor
{

    /**
     * log
     */
    protected static final Logger logger = LoggerFactory.getLogger(HibernateLazyRmiInterceptor.class);

    private SessionFactory sessionFactory;

    /**
     * Contructor
     * @param sessionFactory hibernate session factory
     */
    public HibernateLazyRmiInterceptor(SessionFactory sessionFactory)
    {
        this.sessionFactory = sessionFactory;
    }

    /**
     * {@inheritDoc}
     */
    public Object invoke(MethodInvocation invocation) throws Throwable
    {
        logger.debug("Catched invocation: {}", invocation);
        try
        {
            Object retVal = invocation.proceed();

            SerializationInterceptor.clean();
            // Proxy class for serialization work
            return SerializationInterceptor.getEnhancedObject(retVal, sessionFactory);
        }
        catch (Throwable ex)
        {
            if (logger.isInfoEnabled())
            {
                logger.info("Processing of " + " remote call resulted in exception: ", ex);
            }

            throw ex;
        }
    }

}
