/*
 * Copyright Openmind http://www.openmindonline.it
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package it.openutils.spring.rmibernate.server.exporter;

import it.openutils.spring.rmibernate.server.aspects.HibernateLazyRmiInterceptor;

import org.hibernate.SessionFactory;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.remoting.rmi.RmiServiceExporter;
import org.springframework.remoting.support.RemoteInvocationTraceInterceptor;
import org.springframework.util.ClassUtils;


/**
 * Rmi exporter that checks laziness of bean fields in all object graph
 * @author mmolaschi
 * @version $Id: HibernateRmiServiceExporter.java 716 2008-03-03 14:35:57Z fcarone $
 */
public class HibernateRmiServiceExporter extends RmiServiceExporter
{

    /**
     * hibernate sessionfactory
     */
    private SessionFactory sessionFactory;

    /**
     * {@inheritDoc}
     */
    @Override
    protected Object getProxyForService()
    {
        checkService();
        checkServiceInterface();
        ProxyFactory proxyFactory = new ProxyFactory();
        proxyFactory.addInterface(getServiceInterface());
        if (isRegisterTraceInterceptor())
        {
            proxyFactory.addAdvice(new RemoteInvocationTraceInterceptor(getExporterName()));
        }
        // add interceptor for lazy work
        proxyFactory.addAdvice(new HibernateLazyRmiInterceptor(sessionFactory));
        proxyFactory.setTarget(getService());
        return proxyFactory.getProxy(ClassUtils.getDefaultClassLoader());
    }

    /**
     * Set hibernate session factory
     * @param sessionFactory hibernate sessionfactory
     */
    public void setSessionFactory(SessionFactory sessionFactory)
    {
        this.sessionFactory = sessionFactory;
    }

}
