/*
 * Copyright Openmind http://www.openmindonline.it
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package it.openutils.spring.rmibernate.shared.managers;

import java.io.Serializable;
import java.rmi.RemoteException;


/**
 * Interface for server-side remote lazy loading calls
 * @author mmolaschi
 * @version $Id: HibernateLazyService.java 716 2008-03-03 14:35:57Z fcarone $
 */
public interface HibernateLazyService
{
    /**
     * Invoke remote lazy loading
     * @param className name of parent class
     * @param propertyName name of property to load
     * @param id current entity id
     * @return loaded object / collection
     * @throws RemoteException error
     */
    Object invoke(String className, String propertyName, Serializable id) throws RemoteException;

    /**
     * Invoke remote lazy loading
     * @param className name of parent class
     * @param propertyName name of property to load
     * @param id current entity id
     * @param from starting row number
     * @param size max rows to return
     * @return loaded object / collection
     * @throws RemoteException error
     */
    Object invoke(String className, String propertyName, Serializable id, int from, int size) throws RemoteException;

    /**
     * Get collection size
     * @param className name of parent class
     * @param propertyName name of property to load
     * @param id current entity id
     * @return loaded object / collection
     * @throws RemoteException error
     */
    long count(String className, String propertyName, Serializable id) throws RemoteException;
}
